@@ -1,4 +1,4 @@
-#	$OpenBSD: agent.sh,v 1.10 2014/02/27 21:21:25 djm Exp $
+#	$OpenBSD: agent.sh,v 1.11 2015/03/03 22:35:19 markus Exp $
 #	Placed in the Public Domain.
 
 tid="simple agent test"
@@ -20,7 +20,7 @@ else
 	fi
 	trace "overwrite authorized keys"
 	printf '' > $OBJ/authorized_keys_$USER
-	for t in ed25519 rsa rsa1; do
+	for t in ${SSH_KEYTYPES}; do
 		# generate user key for agent
 		rm -f $OBJ/$t-agent
 		${SSHKEYGEN} -q -N '' -t $t -f $OBJ/$t-agent ||\
@@ -46,7 +46,7 @@ else
 	fi
 
 	trace "simple connect via agent"
-	for p in 1 2; do
+	for p in ${SSH_PROTOCOLS}; do
 		${SSH} -$p -F $OBJ/ssh_proxy somehost exit 5$p
 		r=$?
 		if [ $r -ne 5$p ]; then
@@ -55,7 +55,7 @@ else
 	done
 
 	trace "agent forwarding"
-	for p in 1 2; do
+	for p in ${SSH_PROTOCOLS}; do
 		${SSH} -A -$p -F $OBJ/ssh_proxy somehost ${SSHADD} -l > /dev/null 2>&1
 		r=$?
 		if [ $r -ne 0 ]; then
