@@ -1,4 +1,4 @@
-/* $OpenBSD: uuencode.c,v 1.27 2013/05/17 00:13:14 djm Exp $ */
+/* $OpenBSD: uuencode.c,v 1.28 2015/04/24 01:36:24 deraadt Exp $ */
 /*
  * Copyright (c) 2000 Markus Friedl.  All rights reserved.
  *
@@ -23,6 +23,8 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+#include "includes.h"
+
 #include <sys/types.h>
 #include <netinet/in.h>
 #include <resolv.h>
@@ -80,7 +82,7 @@ dump_base64(FILE *fp, const u_char *data, u_int len)
 		fprintf(fp, "dump_base64: len > 65536\n");
 		return;
 	}
-	buf = xmalloc(2*len);
+	buf = xreallocarray(NULL, 2, len);
 	n = uuencode(data, len, buf, 2*len);
 	for (i = 0; i < n; i++) {
 		fprintf(fp, "%c", buf[i]);
